## controller.py
## ueber den Controller gehen alle Aktionen
## Singleton nach demselben Muster wie bei Leinwand
from Graphenprojekt.knoten import Knoten
from Graphenprojekt.kante import Kante

import time

class _Controller:
    """ vermittelt die Aktionen """
    
    _instance=None

    def __init__(self):
        if _Controller._instance:
            raise Error("Nur ein Controller erlaubt!")
        else:
            _Controller._instance=self
        self.knotenListe = []
        self.kantenListe = []
        self.kantenDict = dict()
        self.namen = [] ## zum Vermeiden von Dopplung

    def gibKantenDict(self): return self.kantenDict

    def gibKante(self,von,nach):
        ''' gibt die Kante zu den Namen zurueck '''
        for kante in self.kantenListe:
            if kante.gibKnotenNamen() == (von,nach):
                return kante
            if kante.gibKnotenNamen() == (nach,von):
                return kante
        return None

    def hatKante(self,vonKnoten,nachKnoten):
        ''' zum Vermeiden von Dopplung '''
        for kante in self.kantenListe:
            if kante.gibKnoten() == (vonKnoten,nachKnoten):
                return True
            if kante.gibKnoten() == (nachKnoten,vonKnoten):
                return True
        return False

    def neuerKnoten(self,name,xPosition=20,yPosition=20):
        if name in self.namen: return
        knoten=Knoten(name,xPosition,yPosition)
        self.knotenListe.append(knoten)
        knoten.zeige()
        return knoten

    def neueKante(self,vonKnoten,nachKnoten,beschriftung=''):
        ''' genau eine Kante zwischen zwei Knoten '''
        if self.hatKante(vonKnoten,nachKnoten): return
        kante=Kante(vonKnoten,nachKnoten,beschriftung)
        self.kantenListe.append(kante)
        kante.zeige()
        return kante

    def loescheKnoten(self,name):
        ''' muss auch Kanten loeschen, die an diesen Knoten haengen '''
        pass

    def lies(self, dateiname):
        '''Verwendet nur die fuer das Modell selbst notwendigen Daten'''
        try:
            f=open(dateiname,'r')
        except IOError as e:
            print('Datei existiert nicht, keine Daten gelesen')
            return None
        graphen_Daten_String=f.read()
        f.close()
        self.__graphenDaten=eval(graphen_Daten_String)
        self.erzeugeGraphenAusDaten(self.__graphenDaten)
        self.erzeugeDict(self.__graphenDaten)
        return self.__graphenDaten

    def erzeugeGraphenAusDaten(self, graphen_Daten):
        for daten in graphen_Daten:
            if daten[0]=='Knoten':
                self.neuerKnoten(daten[1][0],daten[1][1],daten[1][2])
            elif daten[0]=='Kante':
                knoten_1=self.gibKnotenZuNamen(daten[1][0])
                knoten_2=self.gibKnotenZuNamen(daten[1][1])
                laengenString=self.gibZahlString(daten[1][2])
                self.neueKante(knoten_1,knoten_2,laengenString)
            else:
                self.Meldung('Fehler bei Graphendefinition: '+daten[0])

    def erzeugeDict(self, graphen_Daten):
        '''erzeugt die KnotenListe der Kanten als Dictionary'''
        ## initialisieren
        kantenDict = dict()
        for knoten in self.knotenListe:
            kantenDict.update({knoten.gibNamen(): []})
        ## fuellen
        for daten in graphen_Daten:
            if daten[0]=='Kante':
                nachfolger=kantenDict[daten[1][0]]+[[daten[1][1],
                                                     self.gibZahl(daten[1][2])]]
                kantenDict.update({daten[1][0] : nachfolger})
                # und Rueckkanten
                nachfolger=kantenDict[daten[1][1]]+[[daten[1][0],
                                                     self.gibZahl(daten[1][2])]]
                kantenDict.update({daten[1][1] : nachfolger})
        self.kantenDict=kantenDict

    def gibZahlString(self,wert):
        '''gibt den uebergebenen Wert als Zahl zurueck, wenn er sich als
        Zahl generieren laesst'''
        try:
            zahl=float(wert)
        except ValueError as e:
            return ''
        return wert

    def gibZahl(self,wert):
        '''gibt den uebergebenen Wert als Zahl zurueck'''
        try:
            zahl=int(wert)
            return zahl
        except ValueError as e1:
            try:
                zahl=float(wert)
            except ValueError as e2:
                ## kennzeichnet einen Fehler
                return -1

    def gibKnotenZuNamen(self, name):
        for knoten in self.knotenListe:
            if name==knoten.gibNamen():
                return knoten
        return None

    def aendereRandFarbe(self, typ, name, neueFarbe, name_2=''):
        if typ==Knoten:
            Knoten.aendereRandFarbe(self.gibKnotenZuNamen(name),neueFarbe)
        if typ==Kante:
            Kante.aendereRandFarbe(self.gibKante(name,name_2),neueFarbe)
            
    def aendereFuellFarbe(self, typ, name, neueFarbe, name_2=''):
        if typ==Kante or name_2!='':
            return
        Knoten.aendereFuellFarbe(self.gibKnotenZuNamen(name),neueFarbe)

    def warte(self,sec):
        time.sleep(sec)

    def bildExport(self):
        if len(self.knotenListe)>0:
            self.knotenListe[0].leinwand.bildExport()
            
def Controller(singleton=_Controller()):
    """
    Liefert den Controller. Hiervon darf es nur ein einziges Objekt geben,
    deswegen liefert diese Funktion bei jedem Aufruf dasselbe Objekt
    zurück.
    """
    return singleton

